/*********************************************
Chip type           : AT90S2313
Clock frequency     : 4,000000 MHz
Memory model        : Tiny
Internal SRAM size  : 128
External SRAM size  : 0
Data Stack size     : 32
*********************************************/
#include <90s2313.h>
#include <stdlib.h>
#include "main.h"

// Timer 1 output compare interrupt service routine
#pragma	savereg-
interrupt [TIM1_COMP] void timer1_comp_isr(void)
{

// Place your code here
#asm            
	push r26
	push r27
	push r30
	push r31
	in r30,SREG
	push r30
#endasm             
	if(Flags & MotorON){	// if motor enabled
		if(Flags & LeftR){
			StepIndex++;
			if(StepIndex>11) StepIndex = 0;
			}
		else{
			if(!StepIndex) StepIndex = 12;	
			StepIndex--;
			}
                PORTB = StepTable[StepIndex];
		}
	else{
		PORTB = BrakeTable[StepIndex];
		}
#asm               
	pop r30
	out SREG, r30
	pop r31
	pop r30
	pop r27
	pop r26
#endasm
}
interrupt [TIM0_OVF] void timer0_isr(void)
{

// Place your code here
#asm
	push r30
	in r30,SREG
	push r30
#endasm
	TCNT0 -= 198;
	T0IRQcnt++;
	if(!T0IRQcnt){
		Flags |= T0IF;
		}
	if(!PWMcnt){
		PWMcnt = PWMdiscret;
		PWMout = 0;
		}
	if(PWMcnt==PWMval){
		PWMout=1;
		}
	PWMcnt--;
#asm               
	pop r30
	out SREG, r30
	pop r30
#endasm
}
// Declare your global variables here

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=Out Func6=In Func7=In 
// State0=0 State1=0 State2=0 State3=0 State4=0 State5=0 State6=0 State7=0 
PORTB=0x00;
DDRB=0x3F;

// Port D initialization
// Func0=Out Func1=Out Func2=In Func3=In Func4=In Func5=In Func6=Out 
// State0=P State1=P State2=P State3=P State4=P State5=P State6=T 
//PORTD=0x00;
PORTD=0x3F;
DDRD=0x43;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 4 MHz
TCCR0=0x01;
TCNT0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 4000,000 kHz
// Mode: CTC top=OCR1A
// OC1 output: discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;	// Stop Timer1
TCNT1H=0x00;
TCNT1L=0x00;
OCR1H=0xFF;
OCR1L=0xFF;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
GIMSK=0x00;
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x42;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
// Analog Comparator Output: Off
ACSR=0x80;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/1024
//#asm("wdr");
WDTCR=0x0E;
        
// Program Initialisations
Flags = 0;		// clear all flags

rpm = eeprom_rpm;
rpm_min = rpm_low_index;
rpm_max = rpm_high_index;

if(!rpm_min){
	rpm_min = 1;
	rpm_low_index = rpm_min;
	}
if(rpm_max > 15){
	rpm_max = 15;
	rpm_high_index= rpm_max;
	}
if(rpm_min >= rpm_max){
	rpm_min = 3;
	rpm_max = 6;
	rpm_low_index = rpm_min;
	rpm_high_index= rpm_max;
	}
PWMdiscret = eeprom_PWM_discret;
/*
rpm_min = 3;
rpm_max = 6;
PWMdiscret = 64;
rpm = eeprom_rpm;
if((rpm<rpm_min)|(rpm>rpm_max)){
	rpm = rpm_min;
	eeprom_rpm = rpm;
	}
*/
SetRPM(0);		// Disable Motor

if(eeprom_LeftR) Flags |= LeftR;
indic_rpm(rpm);
indic_stat();
print_cnt=0;
printLCD();

TCCR1B=0x09;	// start TMR1
#asm("sei")	// Global enable interrupts

while (1)
      {
#asm("wdr")
      // Place your code here
if (Flags & T0IF){	// Timer1 IF
	T0IFcnt ++;
	Flags &= ~T0IF;
	if(T0IFcnt < T0OVps) continue;
	T0IFcnt=0;

	printLCD();
	if(Flags & MotorON){
		if(!shift_cnt){
			shift_cnt = shift_speed;
			if(rpm!=rpm_last) SetRPM(rpm);
			}
		shift_cnt--;
		}
// -----
	if(save_rpm_cnt){
		save_rpm_cnt--;
		if(!save_rpm_cnt){	// if counter time-outed save rpm to eeprom
			eeprom_rpm = rpm;
			}
		}
	
	if(!LowRkey){	// if Low Rotation pressed
		if(!(Flags & LowRStat)){	// if LowR not pressed yet
			Flags |= LowRStat;	// indicate start low rotation
			SetRPM(rpm_min);
			indic_rpm(rpm_min);	// refresh rpm indication buffer
			indic_stat();		// refresh status
			print_cnt = 0;		// initialise LCD refresh
			}
		continue;
		}
	else if(Flags & LowRStat){		// if was LowR regyme
		Flags &= ~ LowRStat;		// clear LowR flag
		SetRPM(0);			// Stop Motor
		indic_rpm(rpm);			// refresh rpm indication buffer
		indic_stat();			// indicate staus
		}
	tast = getchr();
	if(tast){
		rpm_last = rpm;
		if(tast==keym1){	// DoWn key
			if(rpm<=rpm_min) rpm = rpm_max+1;
			rpm--;
			}
		else if(tast==keym2){	// Up key
			rpm++;
			if(rpm>rpm_max) rpm = rpm_min;
			}
		save_rpm_cnt = TOVps * 10;	// save rpm after 10 sec
		indic_rpm(rpm);			// refresh rpm indication buffer
		print_cnt = 0;			// initialise LCD refresh
		}
	if(!OFFkey){	// if pressed ShotOFF motor
		if(Flags & MotorON){
			SetRPM(0);
			indic_stat();	// refresh status
			print_cnt = 0;	// initialise LCD refresh
			}
		Flags &= ~MotorON;
		}
	else if(!ONkey){
		if(!(Flags & MotorON)){
			SetRPM(rpm);
			indic_stat();	// refresh status
			print_cnt = 0;	// initialise LCD refresh
			}
		}
	if(!LRkey){
		if(!(Flags & DirStat)){	// last status not pressed
			Flags |= DirStat;
			if(Flags & LeftR){      // if left rotation set to right rotation
				Flags &= ~LeftR;	// Change Flag to Right rotaion
				eeprom_LeftR = 0;	// save to eeprom
				}
			else{
				Flags |= LeftR;		// Change Flag to Left rotaion
				eeprom_LeftR = 1;	// save to eeprom			 
			 	}
			indic_stat();
			}
		}
	else	Flags &= ~DirStat;	// set status not pressed
	}
      }
}                                
//_________________________________________________________
// set rpm function
void	SetRPM(char iv){
register unsigned int	li0, li1;
if(iv<rpm_min){
	Flags &= ~MotorON;	// clear MotorON flag
	rpm_last = rpm_min;
	PORTB = 0x00;		// clear ports
	PWMval = 4;
	OCR1 = OCRTable[rpm_min-1];
	}
else{
	li1 = OCRTable[iv-1];		// load Destination value
	li0 = OCR1;			// load StartPoint value
	if(li0 < li1){	// if selected shift_down regyme
		if(li0 == OCRTable[rpm_last-1]){	// if it was a Table value
			OCR1 = (li0 + OCRTable[rpm_last-2])>>1;	// then load a intermediate frequence value
			PWMval = (tab_PWM[rpm_last-1]+
				tab_PWM[rpm_last-2])>>1;		// and PWM value
			}
		else{		// if not a table value jump to next rpm_last value
			rpm_last--;
			OCR1 = OCRTable[rpm_last-1];
			PWMval = tab_PWM[rpm_last-1];
			}
		}
	else if(li0 > li1){	// if selected shift_up
		if(li0 == OCRTable[rpm_last-1]){		// if it was a Table value
			OCR1 = (li0 + OCRTable[rpm_last])>>1;	// then load a intermediate frequence value
			PWMval = (tab_PWM[rpm_last-1]+
				tab_PWM[rpm_last])>>1;		// and PWM value
			}
		else{		// if not a table value jump to next rpm_last value
			OCR1 = OCRTable[rpm_last];
			PWMval = tab_PWM[rpm_last];
			rpm_last++;
			}
		}
	else{
		rpm_last = rpm;
		OCR1 = OCRTable[rpm_last-1];
		PWMval = tab_PWM[rpm_last-1];
		}
	Flags |= MotorON;      		// set MotorON flag
	} // else if(iv<rpm_min)
}
//_________________________________________________________
void		putLCD(char symb_a){
unsigned char	v0;
for (v0 = 0; v0 <= 0xF; v0++) if (symb_a == tab_symb[v0]) break;
symb_a = tab_data[v0];

for (v0 = 8;v0; v0 >>= 1){
	DOpin = 0;			// DI low
	CKpin = 1;			// SK low
	if (symb_a & v0) DOpin = 1;	// DI high
	delay();
	CKpin = 0;			// SK high
	delay();
	}
LCDcmd = 0;
}
//_________________________________________________________
void	printLCD(void){// print to LCD from LCDbuff
register char v0;
	if(!print_cnt){
		for(v0=0;v0<10;v0++)
			putLCD(LCDbuff[v0]);
		print_cnt = printTime;	// refresh after printTime
		}
	print_cnt--;
}
//-----------------
void	indic_rpm(char irpm){	// send to LCDbuff input rpm value
register char lj;		// local LCDbuff index
	// calc LCDbuff index offset to print
if(irpm < rpm_min) irpm = 0;
lj = 0;
if(irpm < 10){
	LCDbuff[0]=' ';
	lj++;
	}
itoa( ((unsigned int) irpm * 100), &LCDbuff[lj]);
LCDbuff[4] = ' ';
}
//-----------------
void	indic_stat(void){	// send to LCDbuff device status
	if(Flags & LeftR) LCDbuff[5] = 'l';
	else		  LCDbuff[5] = 'r';

	LCDbuff[6] = ' ';
	if(Flags & MotorON){
		LCDbuff[7] = ' ';
		LCDbuff[8] = ' ';
		LCDbuff[9] = ' ';
		}
	else{
		LCDbuff[7] = '0';
		LCDbuff[8] = 'F';
		LCDbuff[9] = 'F';
		}
}
//_________________________________________________________
void delay(void){
	#asm("wdr");
//	#asm("nop");
//	#asm("nop");
//	#asm("nop");
//	#asm("nop");
//	#asm("nop");
//	#asm("nop");
	#asm("nop");
	#asm("nop");
}
